//------------------------------------------------------------------------------
// Copyright (c) 2004, Perforce Software, Inc.  All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1.  Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
// 2.  Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------
#define NEED_CHMOD
#include <clientapi.h>
#include <errorlog.h>
#include <vararray.h>
#include <strtable.h>
#include "filelist.h"
#include "adminuser.h"
#include "edituser.h"
#include "revertuser.h"
#include "submituser.h"
#include "syncuser.h"
#include "adminclient.h"


void  
AdminClient::SetArgv( int argc, char *const *argv )
{
    for( int i = 0; i < argc; i++ )
    {
	if( argv[i][0] != '-' ) break;
	if( argv[i][1] && argv[i][1] == 'n' )
	{
	    simulate++;
	    break;
	}
    }
    this->StrDict::SetArgv( argc, argv );
}

//
// First build the dictionary of file umasks, then sync the files, 
// then 
void
AdminClient::Run( const char *func )
{
    StrRef 		cmd( func );
    ClientUser *	ui = 0;
    AdminUser *		aui = 0;

    if( simulate )
    {
	ui = new ClientUser;
    }
    else if( cmd == "edit" )
    {
	ui = aui = new EditUser;
    }
    else if( cmd == "revert" )
    {
	ui = aui = new RevertUser;
    }
    else if( cmd == "sync" )
    {
	ui = aui = new SyncUser;
    }
    else if( cmd == "submit" )
    {
	ui = aui = new SubmitUser;
    }
    else
    {
	ui = new ClientUser;
    }

    // Now run the users command
    ClientApi::Run( func, ui );

    //
    // If the AdminUser class doesn't think we need to do anything, we're 
    // out easy.
    //
    if( !aui || !aui->ChmodFiles() )
	return;

    // run a "p4 fstat" to populate our mode dictionary for those commands 
    // that require it.
    if( aui ) StatFiles( aui, aui->WorkList() );
    
    //
    // Now update permissions 
    //
    SetPerms( aui );
    
}

//
// Pretty ugly method to run the fstat
//
void
AdminClient::StatFiles( ClientUser *ui, FileList *list )
{
    ClientApi c;
    Error	e;

    c.SetUser( GetUser().Text() );
    c.SetPort( GetPort().Text() );
    c.SetClient( GetClient().Text() );
    c.SetHost( GetHost().Text() );
    c.SetCharset( GetCharset().Text() );
    c.SetPassword( GetPassword().Text() );

    c.Init( &e );
    if( e.Test() )
    {
	ErrorLog::Report( &e );
	return;
    }

    VarArray args;
    args.Put( (void*) "-C" );
    args.Put( (void*) "-Oa" );
    args.Put( (void*) "-A" );
    args.Put( (void*) "unixMask" );

    StrBuf *f = 0;
    for( int i = 0; f = list->Get( i ); i++ )
	args.Put( (void*) f->Text() );

    c.SetArgv( args.Count(), (char * const *) args.ElemTab() );
    c.Run( "fstat", ui );
    c.Final( &e );
}

void
AdminClient::SetPerms( AdminUser *ui )
{
    StrDict *	md = ui->Modes();
    StrRef 	file;
    StrRef  	mode;
    Error	e;

    for( int i = 0; md->GetVar( i, file, mode ); i++ )
    {
	ChmodFile( &file, &mode, &e );
	if( e.Test() ) ui->HandleError( &e );
	e.Clear();
    }
}


void
AdminClient::ChmodFile( const StrPtr *file, const StrPtr *mode, Error *e )
{
    int m = strtol( mode->Text(), 0, 0 );
    printf( "... setting mode on %s to %04o\n", file->Text(), m );
    if( chmod( file->Text(), m ) )
	e->Sys( "chmod", file->Text() );
}
