//------------------------------------------------------------------------------
// Copyright (c) 2004, Perforce Software, Inc.  All rights reserved.
// 
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are met:
// 
// 1.  Redistributions of source code must retain the above copyright
//     notice, this list of conditions and the following disclaimer.
// 
// 2.  Redistributions in binary form must reproduce the above copyright
//     notice, this list of conditions and the following disclaimer in the
//     documentation and/or other materials provided with the distribution.
// 
// THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
// AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
// ARE DISCLAIMED. IN NO EVENT SHALL PERFORCE SOFTWARE, INC. BE LIABLE FOR ANY
// DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
// (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
// LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
// ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
// (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//------------------------------------------------------------------------------
#include <clientapi.h>
#include <strtable.h>
#include <vararray.h>
#include "filelist.h"
#include "revertuser.h"

static StrRef	msgRevert1( "%depotFile%%haveRev% - was %action%, unlocked and reverted" );
static StrRef	msgRevert2( "%depotFile%%haveRev% - was %action%, reverted" );

//
// Only called when we run the revert. For each output message, find the depot
// path in the message (if possible), and append it to the worklist. 
//
void
RevertUser::Message( Error *err )
{
    // Ensure default behaviour happens
    ClientUser::Message( err );

    if( !err->IsInfo() ) return;
    
    StrPtr *	depotFile	= varList->GetVar( "depotFile" );
    StrPtr *	haveRev		= varList->GetVar( "haveRev" );
    StrBuf	file;
    
    if( msgRevert1 == err->GetId( 0 )->fmt || 
	msgRevert2 == err->GetId( 0 )->fmt    )
    {
	if( depotFile && haveRev )
	{
	    file << *depotFile << *haveRev;
	    PushFile( &file );
	}
    }
}

//
// Handle errors during edits
//
void
RevertUser::HandleError( Error *e )
{
    ClientUser::HandleError( e );
    if( !e->IsInfo() )
	PopFile();
}

//
// Only chmod files if we've got something in the worklist
//
int
RevertUser::ChmodFiles()
{
    return WorkList()->Count();
}

